<?php

namespace SpkS\Package;

/**
 * SPK Package class
 *
 * @property string $spk Path to SPK file
 * @property string $spk_url URL to SPK file
 * @property string $displayname Pretty printed name of package (falls back to $package if not present)
 * @property string $package Package name
 * @property string $version Package version
 * @property string $description Package description
 * @property string $maintainer Package maintainer
 * @property string $maintainer_url URL of maintainer's web page
 * @property string $distributor Package distributor
 * @property string $distributor_url URL of distributor's web page
 * @property array $arch List of supported architectures, or 'noarch'
 * @property array $thumbnail List of thumbnail files
 * @property array $thumbnail_url List of thumbnail URLs
 * @property array $snapshot List of screenshot files
 * @property array $snapshot_url List of screenshot URLs
 * @property bool $beta TRUE if this is a beta package.
 * @property bool $signed TRUE if this is a signed package.
 * @property string $firmware Minimum firmware needed on device.
 * @property string $install_dep_services Dependencies required by this package.
 * @property bool $silent_install Allow silent install
 * @property bool $silent_uninstall Allow silent uninstall
 * @property bool $silent_upgrade Allow silent upgrade
 */
class Package
{
    private $config;
    private $filepath;
    private $filepathNoExt;
    private $filename;
    private $filenameNoExt;
    private $metafile;
    private $metadata;

    /**
     * @param \SpkS\Config $config Config object
     * @param string $filename Filename of SPK file
     */
    public function __construct(\SpkS\Config $config, $filename)
    {
        $this->config = $config;
        if (!preg_match('/\.spk$/', $filename)) {
            throw new \Exception('文件 ' . $filename . ' 没有 .spk 扩展名!');
        }
        if (!file_exists($filename)) {
            throw new \Exception('文件 ' . $filename . ' 未找到!');
        }
        $this->filepath      = $filename;
        $this->filename      = basename($filename);
        $this->filenameNoExt = basename($filename, '.spk');
        $this->filepathNoExt = $this->config->paths['cache'] . $this->filenameNoExt;
        $this->metafile      = $this->filepathNoExt . '.nfo';
    }

    /**
     * Getter magic method.
     *
     * @param string $name Name of requested value.
     * @return mixed Requested value.
     */
    public function __get($name)
    {
        $this->collectMetadata();
        return $this->metadata[$name];
    }

    /**
     * Setter magic method.
     *
     * @param string $name Name of variable to set.
     * @param mixed $value Value to set.
     */
    public function __set($name, $value)
    {
        $this->collectMetadata();
        $this->metadata[$name] = $value;
    }

    /**
     * Isset feature magic method.
     *
     * @param string $name Name of requested value.
     * @return bool TRUE if value exists, FALSE otherwise.
     */
    public function __isset($name)
    {
        $this->collectMetadata();
        return isset($this->metadata[$name]);
    }

    /**
     * Unset feature magic method.
     *
     * @param string $name Name of value to unset.
     */
    public function __unset($name)
    {
        $this->collectMetadata();
        unset($this->metadata[$name]);
    }

    /**
     * Parses boolean value ('yes', '1', 'true') into
     * boolean type.
     *
     * @param mixed $value Input value
     * @return bool Boolean interpretation of $value.
     */
    public function parseBool($value)
    {
        return in_array($value, array('true', 'yes', '1', 1));
    }

    /**
     * Checks if given property $prop exists and converts it
     * into a boolean value.
     *
     * @param string $prop Property to convert
     */
    private function fixBoolIfExist($prop)
    {
        if (isset($this->metadata[$prop])) {
            $this->metadata[$prop] = $this->parseBool($this->metadata[$prop]);
        }
    }

    /**
     * Gathers metadata from package. Extracts INFO file if neccessary.
     */
    private function collectMetadata()
    {
        if (!is_null($this->metadata)) {
            // metadata already collected
            return;
        }
		
		$metadate = date("YmdHis", filemtime($this->metafile));
		$filedate  = date("YmdHis", filemtime($this->filepath));
		if ($metadate < $filedate) {
			//unlink($this->metafile);
			$this->deleteAllFiles($this->filepathNoExt . '.asc');
			$this->deleteAllFiles($this->filepathNoExt . '_thumb_*.png');
			$this->deleteAllFiles($this->filepathNoExt . '_screen_*.png');
		}
		
        $this->extractIfMissing('INFO', $this->metafile);
        $this->metadata = parse_ini_file($this->metafile);
        if (!isset($this->metadata['displayname'])) {
            $this->metadata['displayname'] = $this->metadata['package'];
        }
        $this->metadata['spk'] = $this->filepath;

        // Convert architecture(s) to array, as multiple architectures can be specified
        $this->metadata['arch'] = explode(' ', $this->metadata['arch']);

        $this->fixBoolIfExist('silent_install');
        $this->fixBoolIfExist('silent_uninstall');
        $this->fixBoolIfExist('silent_upgrade');

        if (isset($this->metadata['beta']) && in_array($this->metadata['beta'], array('true', 'yes', '1', 'beta'))) {
            $this->metadata['beta'] = true;
        } else {
            $this->metadata['beta'] = false;
        }

        $this->metadata['thumbnail'] = $this->getThumbnails();
        $this->metadata['snapshot'] = $this->getSnapshots();
		$this->metadata['signed'] = $this->getSignature();
    }

	public function deleteAllFiles($path)
	{
		$files = glob($path);
		foreach($files as $file)
		{
			unlink($file);		
		}
		return true;
	}
	
    /**
     * Returns metadata for this package.
     *
     * @return array Metadata.
     */
    public function getMetadata()
    {
        $this->collectMetadata();
        return $this->metadata;
    }

    /**
     * Extracts $inPkgName from package to $targetFile, if it doesn't
     * already exist. Needs the phar.so extension and allow_url_fopen.
     *
     * @param string $inPkgName Filename in package
     * @param string $targetFile Path to destination
     * @throws \Exception if the file couldn't get extracted.
     * @return bool TRUE if successful or no action needed.
     */
    public function extractIfMissing($inPkgName, $targetFile)
    {
        if (file_exists($targetFile)) {
            // Everything in working order
            return true;
        }
        // Try to extract file
        $tmp_dir = sys_get_temp_dir();
        $free_tmp = disk_free_space($tmp_dir);
        if ($free_tmp < 2048) {
            throw new \Exception('TMP 文件夹只有 ' . $free_tmp . ' 字节可用空间。磁盘已满!');
        }
        $free = disk_free_space(dirname($targetFile));
        if ($free < 2048) {
            throw new \Exception('套件包文件夹只有 ' . $free . ' 字节可用空间。磁盘已满!');
        }
        try {
            $p = new \PharData($this->filepath, \Phar::CURRENT_AS_FILEINFO | \Phar::KEY_AS_FILENAME);
        } catch (\UnexpectedValueException $e) {
            rename($this->filepath, $this->filepath . '.invalid');
            throw new \Exception('套件包' . $this->filepath . ' 不可读! 将会被忽略。请再试一次!');
        }
        $tmpExtractedFilepath = $tmp_dir . DIRECTORY_SEPARATOR . $inPkgName;
        if (file_exists($tmpExtractedFilepath)) {
            // stale file from before - unlink first
            unlink($tmpExtractedFilepath);
        }
        $p->extractTo($tmp_dir, $inPkgName);
        rename($tmpExtractedFilepath, $targetFile);
        return true;
    }

    /**
     * Returns a list of thumbnails for the specified package.
     *
     * @param string $pathPrefix Prefix to put before file path
     * @return array List of thumbnail urls
     */
    public function getThumbnails($pathPrefix = '')
    {
        $thumbnailSources = array(
            '72' => array(
                'file' => 'PACKAGE_ICON.PNG',
                'info' => 'package_icon',
            ),
            '120' => array(
                'file' => 'PACKAGE_ICON_256.PNG',
                'info' => 'package_icon_256',
            ),
        );
        $thumbnails = array();
        foreach ($thumbnailSources as $size => $sourceList) {
            $thumbName = $this->filepathNoExt . '_thumb_' . $size . '.png';
            // Try to find file in package, otherwise check if defined in INFO
            try {
                $this->extractIfMissing($sourceList['file'], $thumbName);
            } catch (\Exception $e) {
                // Check if icon is in metadata
                $this->collectMetadata();
                if (isset($this->metadata[$sourceList['info']])) {
                    file_put_contents($thumbName, base64_decode($this->metadata[$sourceList['info']]));
                }
            }

            // Use $size px thumbnail, if available
            if (file_exists($thumbName)) {
                $thumbnails[] = $pathPrefix . $thumbName;
            } else {
                // Use theme's default pictures
                $themeUrl = $this->config->paths['themes'] . $this->config->site['theme'] . '/';
                $thumbnails[] = $pathPrefix . $themeUrl . 'images/default_package_icon_' . $size . '.png';
            }
        }
        return $thumbnails;
    }

    /**
     * Returns a list of screenshots for the specified package.
     *
     * @param string $pathPrefix Prefix to put before file path
     * @return array List of screenshots
     */
    public function getSnapshots($pathPrefix = '')
    {
        /* Let's first try to extract screenshots from package (SpkS feature) */
        $i = 1;
        while (true) {
            try {
                $this->extractIfMissing('screen_' . $i . '.png', $this->filepathNoExt . '_screen_' . $i . '.png');
                $i++;
            } catch (\Exception $e) {
                break;
            }
        }
        $snapshots = array();
        // Add screenshots, if available
        foreach (glob($this->filepathNoExt . '*_screen_*.png') as $snapshot) {
            $snapshots[] = $pathPrefix . $snapshot;
        }		
        return $snapshots;
    }

    /**
     * Check if the package has a signature file.
     *
     * @return true or false
     */
    public function getSignature()
    {
	$signature = $this->filepathNoExt . '.asc';
	try {
		$this->extractIfMissing('syno_signature.asc', $signature);
	} catch (\Exception $e) {			
	}
		
        return file_exists($signature);
    }
	
    /**
     * Checks compatibility to the given $arch-itecture.
     *
     * @param string $arch Architecture to check against (or "noarch")
     * @return bool TRUE if compatible, otherwise FALSE.
     */
    public function isCompatibleToArch($arch)
    {
        // Make sure we have metadata available
        $this->collectMetadata();
        // TODO: Check arch family, too?
        return (in_array($arch, $this->metadata['arch']) || in_array('noarch', $this->metadata['arch']));
    }

    /**
     * Checks compatibility to the given firmware $version.
     *
     * @param string $version Target firmware version.
     * @return bool TRUE if compatible, otherwise FALSE.
     */
    public function isCompatibleToFirmware($version)
    {
        $this->collectMetadata();
        return version_compare($this->metadata['firmware'], $version, '<=');
    }

    /**
     * Checks if this package is a beta version or not.
     *
     * @return bool TRUE if this is a beta version, FALSE otherwise.
     */
    public function isBeta()
    {
        $this->collectMetadata();
        return (isset($this->metadata['beta']) && $this->metadata['beta'] == true);
    }
	
    /**
     * Checks if this package is signed or not.
     *
     * @return bool TRUE if this is signed, FALSE otherwise.
     */
    public function isSigned()
    {
        $this->collectMetadata();
        return (isset($this->metadata['signed']) && $this->metadata['signed'] == true);
    }	
}